@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -49,7 +55,7 @@ public:
         : q_ptr(q),
           sortField(QQuickFolderListModel::Name), sortReversed(false), showFiles(true),
           showDirs(true), showDirsFirst(false), showDotAndDotDot(false), showOnlyReadable(false),
-          showHidden(false)
+          showHidden(false), caseSensitive(true)
     {
         nameFilters << QLatin1String("*");
     }
@@ -70,6 +76,7 @@ public:
     bool showDotAndDotDot;
     bool showOnlyReadable;
     bool showHidden;
+    bool caseSensitive;
 
     ~QQuickFolderListModelPrivate() {}
     void init();
@@ -440,9 +447,9 @@ void QQuickFolderListModel::setFolder(const QUrl &folder)
 /*!
    \qmlproperty url FolderListModel::rootFolder
 
-   When this property is set, the given folder will
-   be treated as the root in the file system, so that
-   you can only traverse subfolders within it.
+   When the rootFolder is set, then this folder will
+   be threated as the root in the file system, so that
+   you can only travers sub folders from this rootFolder.
 */
 QUrl QQuickFolderListModel::rootFolder() const
 {
@@ -762,6 +769,30 @@ void QQuickFolderListModel::setShowOnlyReadable(bool on)
     }
 }
 
+/*!
+ * \qmlproperty bool FolderListModel::caseSensitive
+ * \since 5.7
+ *
+ * Use case sensitive pattern matching.
+ *
+ * By default, this property is true.
+ *
+ */
+bool QQuickFolderListModel::caseSensitive() const
+{
+    Q_D(const QQuickFolderListModel);
+    return d->caseSensitive;
+}
+
+void QQuickFolderListModel::setCaseSensitive(bool on)
+{
+    Q_D(QQuickFolderListModel);
+
+    if (on != d->caseSensitive) {
+        d->fileInfoThread.setCaseSensitive(on);
+    }
+}
+
 /*!
     \qmlmethod var FolderListModel::get(int index, string property)
 
