@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -40,12 +46,10 @@
 
 #include <private/qqmlcustomparser_p.h>
 #include <private/qqmlengine_p.h>
-#include <private/qqmlnotifier_p.h>
 
 #include <private/qv4object_p.h>
 #include <private/qv4dateobject_p.h>
 #include <private/qv4objectiterator_p.h>
-#include <private/qv4alloca_p.h>
 
 #include <qqmlcontext.h>
 #include <qqmlinfo.h>
@@ -54,7 +58,6 @@
 #include <QtCore/qstack.h>
 #include <QXmlStreamReader>
 #include <QtCore/qdatetime.h>
-#include <QScopedValueRollback>
 
 QT_BEGIN_NAMESPACE
 
@@ -166,9 +169,7 @@ ListLayout::ListLayout(const ListLayout *other) : currentBlock(0), currentBlockO
 
 ListLayout::~ListLayout()
 {
-    for (int i=0 ; i < roles.count() ; ++i) {
-        delete roles[i];
-    }
+    qDeleteAll(roles);
 }
 
 void ListLayout::sync(ListLayout *src, ListLayout *target)
@@ -395,8 +396,6 @@ void ListModel::updateCacheIndices()
 
 QVariant ListModel::getProperty(int elementIndex, int roleIndex, const QQmlListModel *owner, QV4::ExecutionEngine *eng)
 {
-    if (roleIndex >= m_layout->roleCount())
-        return QVariant();
     ListElement *e = elements[elementIndex];
     const ListLayout::Role &r = m_layout->getExistingRole(roleIndex);
     return e->getProperty(r, owner, eng);
@@ -1262,14 +1261,9 @@ ModelNodeMetaObject *ModelNodeMetaObject::get(QObject *obj)
 
 void ModelNodeMetaObject::updateValues()
 {
-    const int roleCount = m_model->m_listModel->roleCount();
-    if (!m_initialized) {
-        int *changedRoles = reinterpret_cast<int *>(alloca(roleCount * sizeof(int)));
-        for (int i = 0; i < roleCount; ++i)
-            changedRoles[i] = i;
-        emitDirectNotifies(changedRoles, roleCount);
+    if (!m_initialized)
         return;
-    }
+    int roleCount = m_model->m_listModel->roleCount();
     for (int i=0 ; i < roleCount ; ++i) {
         const ListLayout::Role &role = m_model->m_listModel->getExistingRole(i);
         QByteArray name = role.name.toUtf8();
@@ -1280,10 +1274,8 @@ void ModelNodeMetaObject::updateValues()
 
 void ModelNodeMetaObject::updateValues(const QVector<int> &roles)
 {
-    if (!m_initialized) {
-        emitDirectNotifies(roles.constData(), roles.count());
+    if (!m_initialized)
         return;
-    }
     int roleCount = roles.count();
     for (int i=0 ; i < roleCount ; ++i) {
         int roleIndex = roles.at(i);
@@ -1313,22 +1305,6 @@ void ModelNodeMetaObject::propertyWritten(int index)
     }
 }
 
-// Does the emission of the notifiers when we haven't created the meta-object yet
-void ModelNodeMetaObject::emitDirectNotifies(const int *changedRoles, int roleCount)
-{
-    Q_ASSERT(!m_initialized);
-    QQmlData *ddata = QQmlData::get(object(), /*create*/false);
-    if (!ddata)
-        return;
-    QQmlEnginePrivate *ep = QQmlEnginePrivate::get(qmlEngine(m_model));
-    if (!ep)
-        return;
-    for (int i = 0; i < roleCount; ++i) {
-        const int changedRole = changedRoles[i];
-        QQmlNotifier::notify(ddata, changedRole);
-    }
-}
-
 namespace QV4 {
 
 void ModelObject::put(Managed *m, String *name, const Value &value)
@@ -1358,18 +1334,6 @@ ReturnedValue ModelObject::get(const Managed *m, String *name, bool *hasProperty
         return QObjectWrapper::get(m, name, hasProperty);
     if (hasProperty)
         *hasProperty = true;
-
-    if (QQmlEngine *qmlEngine = that->engine()->qmlEngine()) {
-        QQmlEnginePrivate *ep = QQmlEnginePrivate::get(qmlEngine);
-        if (ep && ep->propertyCapture) {
-            QObjectPrivate *op = QObjectPrivate::get(that->object());
-            // Temporarily hide the dynamic meta-object, to prevent it from being created when the capture
-            // triggers a QObject::connectNotify() by calling obj->metaObject().
-            QScopedValueRollback<QDynamicMetaObjectData*> metaObjectBlocker(op->metaObject, 0);
-            ep->propertyCapture->captureProperty(that->object(), -1, role->index);
-        }
-    }
-
     const int elementIndex = that->d()->m_elementIndex;
     QVariant value = that->d()->m_model->data(elementIndex, role->index);
     return that->engine()->fromVariant(value);
@@ -1439,13 +1403,8 @@ void DynamicRoleModelNode::sync(DynamicRoleModelNode *src, DynamicRoleModelNode
 
 void DynamicRoleModelNode::updateValues(const QVariantMap &object, QVector<int> &roles)
 {
-    const QList<QString> &keys = object.keys();
-
-    QList<QString>::const_iterator it = keys.begin();
-    QList<QString>::const_iterator end = keys.end();
-
-    while (it != end) {
-        const QString &key = *it;
+    for (auto it = object.cbegin(), end = object.cend(); it != end; ++it) {
+        const QString &key = it.key();
 
         int roleIndex = m_owner->m_roles.indexOf(key);
         if (roleIndex == -1) {
@@ -1453,7 +1412,7 @@ void DynamicRoleModelNode::updateValues(const QVariantMap &object, QVector<int>
             m_owner->m_roles.append(key);
         }
 
-        QVariant value = object[key];
+        QVariant value = it.value();
 
         // A JS array/object is translated into a (hierarchical) QQmlListModel,
         // so translate to a variant map/list first with toVariant().
@@ -1484,8 +1443,6 @@ void DynamicRoleModelNode::updateValues(const QVariantMap &object, QVector<int>
 
         if (m_meta->setValue(keyUtf8, value))
             roles << roleIndex;
-
-        ++it;
     }
 }
 
@@ -1703,8 +1660,7 @@ QQmlListModel::QQmlListModel(QQmlListModel *orig, QQmlListModelWorkerAgent *agen
 
 QQmlListModel::~QQmlListModel()
 {
-    for (int i=0 ; i < m_modelObjects.count() ; ++i)
-        delete m_modelObjects[i];
+    qDeleteAll(m_modelObjects);
 
     if (m_primary) {
         m_listModel->destroy();
@@ -2048,8 +2004,7 @@ void QQmlListModel::clear()
     emitItemsAboutToBeRemoved(0, cleared);
 
     if (m_dynamicRoles) {
-        for (int i=0 ; i < m_modelObjects.count() ; ++i)
-            delete m_modelObjects[i];
+        qDeleteAll(m_modelObjects);
         m_modelObjects.clear();
     } else {
         m_listModel->clear();
