@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -257,6 +263,56 @@ Item {
     \endqml
 */
 
+/*!
+    \qmlproperty int QtQuick::MouseEvent::source
+    \since 5.7
+
+    This property holds the source of the mouse event.
+
+    The mouse event source can be used to distinguish between genuine and
+    artificial mouse events. When using other pointing devices such as
+    touchscreens and graphics tablets, if the application does not make use of
+    the actual touch or tablet events, mouse events may be synthesized by the
+    operating system or by Qt itself.
+
+    The value can be one of:
+
+    \value Qt.MouseEventNotSynthesized The most common value. On platforms where
+    such information is available, this value indicates that the event
+    represents a genuine mouse event from the system.
+
+    \value Qt.MouseEventSynthesizedBySystem Indicates that the mouse event was
+    synthesized from a touch or tablet event by the platform.
+
+    \value Qt.MouseEventSynthesizedByQt Indicates that the mouse event was
+    synthesized from an unhandled touch or tablet event by Qt.
+
+    \value Qt.MouseEventSynthesizedByApplication Indicates that the mouse event
+    was synthesized by the application. This allows distinguishing
+    application-generated mouse events from the ones that are coming from the
+    system or are synthesized by Qt.
+
+    For example, to react only to events which come from an actual mouse:
+    \qml
+    MouseArea {
+        onPressed: if (mouse.source !== Qt.MouseEventNotSynthesized) {
+            mouse.accepted = false
+        }
+
+        onClicked: doSomething()
+    }
+    \endqml
+
+    If the handler for the press event rejects the event, it will be propagated
+    further, and then another Item underneath can handle synthesized events
+    from touchscreens. For example, if a Flickable is used underneath (and the
+    MouseArea is not a child of the Flickable), it can be useful for the
+    MouseArea to handle genuine mouse events in one way, while allowing touch
+    events to fall through to the Flickable underneath, so that the ability to
+    flick on a touchscreen is retained. In that case the ability to drag the
+    Flickable via mouse would be lost, but it does not prevent Flickable from
+    receiving mouse wheel events.
+*/
 
 /*!
     \qmltype WheelEvent
@@ -323,7 +379,7 @@ Item {
     \qmlproperty point QtQuick::WheelEvent::pixelDelta
 
     This property holds the delta in screen pixels and is available in platforms that
-    have high-resolution trackpads, such as \macos.
+    have high-resolution trackpads, such as OS X.
     The x and y cordinate of this property holds the delta in horizontal and
     vertical orientation. The value should be used directly to scroll content on screen.
 
