@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -194,27 +200,6 @@ QPointF QQuickParentAnimationPrivate::computeTransformOrigin(QQuickItem::Transfo
     }
 }
 
-struct QQuickParentAnimationData : public QAbstractAnimationAction
-{
-    QQuickParentAnimationData() : reverse(false) {}
-    ~QQuickParentAnimationData() { qDeleteAll(pc); }
-
-    QQuickStateActions actions;
-    //### reverse should probably apply on a per-action basis
-    bool reverse;
-    QList<QQuickParentChange *> pc;
-    void doAction() Q_DECL_OVERRIDE
-    {
-        for (int ii = 0; ii < actions.count(); ++ii) {
-            const QQuickStateAction &action = actions.at(ii);
-            if (reverse)
-                action.event->reverse();
-            else
-                action.event->execute();
-        }
-    }
-};
-
 QAbstractAnimationJob* QQuickParentAnimation::transition(QQuickStateActions &actions,
                         QQmlProperties &modified,
                         TransitionDirection direction,
@@ -222,6 +207,27 @@ QAbstractAnimationJob* QQuickParentAnimation::transition(QQuickStateActions &act
 {
     Q_D(QQuickParentAnimation);
 
+    struct QQuickParentAnimationData : public QAbstractAnimationAction
+    {
+        QQuickParentAnimationData() : reverse(false) {}
+        ~QQuickParentAnimationData() { qDeleteAll(pc); }
+
+        QQuickStateActions actions;
+        //### reverse should probably apply on a per-action basis
+        bool reverse;
+        QList<QQuickParentChange *> pc;
+        void doAction() Q_DECL_OVERRIDE
+        {
+            for (int ii = 0; ii < actions.count(); ++ii) {
+                const QQuickStateAction &action = actions.at(ii);
+                if (reverse)
+                    action.event->reverse();
+                else
+                    action.event->execute();
+            }
+        }
+    };
+
     QQuickParentAnimationData *data = new QQuickParentAnimationData;
     QQuickParentAnimationData *viaData = new QQuickParentAnimationData;
 
@@ -836,12 +842,13 @@ QAbstractAnimationJob* QQuickPathAnimation::transition(QQuickStateActions &actio
         prevData = *d->activeAnimations[target]->pathUpdater();
     }
 
-    QList<QQuickItem*> keys = d->activeAnimations.keys();
-    foreach (QQuickItem *item, keys) {
-        QQuickPathAnimationAnimator *anim = d->activeAnimations.value(item);
+    for (auto it = d->activeAnimations.begin(); it != d->activeAnimations.end();) {
+        QQuickPathAnimationAnimator *anim = it.value();
         if (anim->state() == QAbstractAnimationJob::Stopped) {
             anim->clearTemplate();
-            d->activeAnimations.remove(item);
+            it = d->activeAnimations.erase(it);
+        } else {
+            ++it;
         }
     }
 
