@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -652,13 +658,10 @@ bool QQuickStyledTextPrivate::parseAnchorAttributes(const QChar *&ch, const QStr
 void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QString &textIn, QString &textOut)
 {
     qreal imgWidth = 0.0;
-    QFontMetricsF fm(layout.font());
-    const qreal spaceWidth = fm.width(QChar::Nbsp);
-    const bool trailingSpace = textOut.endsWith(space);
 
     if (!updateImagePositions) {
         QQuickStyledTextImgTag *image = new QQuickStyledTextImgTag;
-        image->position = textOut.length() + (trailingSpace ? 0 : 1);
+        image->position = textOut.length() + 1;
 
         QPair<QStringRef,QStringRef> attr;
         do {
@@ -695,16 +698,14 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
         }
 
         imgWidth = image->size.width();
-        image->offset = -std::fmod(imgWidth, spaceWidth) / 2.0;
         imgTags->append(image);
 
     } else {
         // if we already have a list of img tags for this text
         // we only want to update the positions of these tags.
         QQuickStyledTextImgTag *image = imgTags->value(nbImages);
-        image->position = textOut.length() + (trailingSpace ? 0 : 1);
+        image->position = textOut.length() + 1;
         imgWidth = image->size.width();
-        image->offset = -std::fmod(imgWidth, spaceWidth) / 2.0;
         QPair<QStringRef,QStringRef> attr;
         do {
             attr = parseAttribute(ch, textIn);
@@ -712,9 +713,9 @@ void QQuickStyledTextPrivate::parseImageAttributes(const QChar *&ch, const QStri
         nbImages++;
     }
 
-    QString padding(qFloor(imgWidth / spaceWidth), QChar::Nbsp);
-    if (!trailingSpace)
-        textOut += QLatin1Char(' ');
+    QFontMetricsF fm(layout.font());
+    QString padding(qFloor(imgWidth / fm.width(QChar::Nbsp)), QChar::Nbsp);
+    textOut += QLatin1Char(' ');
     textOut += padding;
     textOut += QLatin1Char(' ');
 }
