@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -66,7 +61,6 @@ private slots:
     void qtbug_22535();
     void evalAfterInvalidate();
     void qobjectDerived();
-    void qtbug_49232();
 
 private:
     QQmlEngine engine;
@@ -206,8 +200,6 @@ class TestObject : public QObject
     Q_PROPERTY(int a READ a NOTIFY aChanged)
     Q_PROPERTY(int b READ b NOTIFY bChanged)
     Q_PROPERTY(int c READ c NOTIFY cChanged)
-    Q_PROPERTY(char d READ d NOTIFY dChanged)
-    Q_PROPERTY(uchar e READ e NOTIFY eChanged)
 
 public:
     TestObject() : _a(10), _b(10), _c(10) {}
@@ -221,25 +213,15 @@ public:
     int c() const { return _c; }
     void setC(int c) { _c = c; emit cChanged(); }
 
-    char d() const { return _d; }
-    void setD(char d) { _d = d; emit dChanged(); }
-
-    uchar e() const { return _e; }
-    void setE(uchar e) { _e = e; emit eChanged(); }
-
 signals:
     void aChanged();
     void bChanged();
     void cChanged();
-    void dChanged();
-    void eChanged();
 
 private:
     int _a;
     int _b;
     int _c;
-    char _d;
-    uchar _e;
 };
 
 #define TEST_CONTEXT_PROPERTY(ctxt, name, value) \
@@ -712,22 +694,6 @@ void tst_qqmlcontext::qobjectDerived()
     QCOMPARE(command.count, 2);
 }
 
-void tst_qqmlcontext::qtbug_49232()
-{
-    TestObject testObject;
-    testObject.setD('a');
-    testObject.setE(97);
-
-    QQmlEngine engine;
-    engine.rootContext()->setContextProperty("TestObject", &testObject);
-    QQmlComponent component(&engine);
-    component.setData("import QtQuick 2.0; QtObject { property int valueOne: TestObject.d; property int valueTwo: TestObject.e }", QUrl());
-    QScopedPointer<QObject> obj(component.create());
-
-    QCOMPARE(obj->property("valueOne"), QVariant('a'));
-    QCOMPARE(obj->property("valueTwo"), QVariant(97));
-}
-
 QTEST_MAIN(tst_qqmlcontext)
 
 #include "tst_qqmlcontext.moc"
