@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -94,7 +89,6 @@ private slots:
     void customValueTypeInQml();
     void gadgetInheritance();
     void toStringConversion();
-    void enumProperties();
 
 private:
     QQmlEngine engine;
@@ -326,6 +320,7 @@ void tst_qqmlvaluetypes::locale()
         QScopedPointer<QObject> object(component.create());
         QVERIFY(!object.isNull());
 
+#ifndef QT_NO_IM
         QVERIFY(QQml_guiProvider()->inputMethod());
         QInputMethod *inputMethod = qobject_cast<QInputMethod*>(QQml_guiProvider()->inputMethod());
         QLocale locale = inputMethod->locale();
@@ -352,6 +347,7 @@ void tst_qqmlvaluetypes::locale()
         }
         QCOMPARE(weekDays, locale.weekdays());
         QCOMPARE(object->property("zeroDigit").toString().at(0), locale.zeroDigit());
+#endif // QT_NO_IM
     }
 }
 
@@ -1653,39 +1649,6 @@ void tst_qqmlvaluetypes::toStringConversion()
     QCOMPARE(stringConversion.toString(), StringLessGadget_to_QString(g));
 }
 
-struct GadgetWithEnum
-{
-    Q_GADGET
-public:
-
-    enum MyEnum { FirstValue, SecondValue };
-
-    Q_ENUM(MyEnum)
-    Q_PROPERTY(MyEnum enumProperty READ enumProperty)
-
-    MyEnum enumProperty() const { return SecondValue; }
-};
-
-void tst_qqmlvaluetypes::enumProperties()
-{
-    QJSEngine engine;
-
-    // When creating the property cache for the gadget when MyEnum is _not_ a registered
-    // meta-type, then QMetaProperty::type() will return QMetaType::Int and consequently
-    // property-read meta-calls will return an int (as expected in this test). However if we
-    // explicitly register the gadget, then QMetaProperty::type() will return the user-type
-    // and QQmlValueTypeWrapper should still handle that and return an integer/number for the
-    // enum property when it is read.
-    qRegisterMetaType<GadgetWithEnum::MyEnum>();
-
-    GadgetWithEnum g;
-    QJSValue value = engine.toScriptValue(g);
-
-    QJSValue enumValue = value.property("enumProperty");
-    QVERIFY(enumValue.isNumber());
-    QCOMPARE(enumValue.toInt(), int(g.enumProperty()));
-}
-
 
 QTEST_MAIN(tst_qqmlvaluetypes)
 
