@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -43,16 +30,19 @@
 #include <QtTest/QSignalSpy>
 #include <private/qquickmultipointtoucharea_p.h>
 #include <private/qquickflickable_p.h>
+#include <private/qquickmousearea_p.h>
 #include <qpa/qwindowsysteminterface.h>
 #include <QtQuick/qquickview.h>
 #include <QtGui/QScreen>
 #include "../../shared/util.h"
+#include "../shared/viewtestutil.h"
 
 class tst_QQuickMultiPointTouchArea : public QQmlDataTest
 {
     Q_OBJECT
 public:
     tst_QQuickMultiPointTouchArea() : device(0) { }
+
 private slots:
     void initTestCase() {
         QQmlDataTest::initTestCase();
@@ -72,9 +62,13 @@ private slots:
     void nested();
     void inFlickable();
     void inFlickable2();
+    void inMouseArea();
+    void mouseAsTouchpoint();
     void invisible();
     void transformedTouchArea_data();
     void transformedTouchArea();
+    void mouseInteraction();
+    void mouseInteraction_data();
 
 private:
     QQuickView *createAndShowView(const QString &file);
@@ -113,6 +107,7 @@ void tst_QQuickMultiPointTouchArea::signalTest()
     QTest::QTouchEventSequence sequence = QTest::touchEvent(window.data(), device);
 
     sequence.press(0, p1).press(1, p2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(area->property("touchPointPressCount").toInt(), 2);
     QCOMPARE(area->property("touchPointUpdateCount").toInt(), 0);
@@ -121,6 +116,7 @@ void tst_QQuickMultiPointTouchArea::signalTest()
     QMetaObject::invokeMethod(area, "clearCounts");
 
     sequence.stationary(0).stationary(1).press(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(area->property("touchPointPressCount").toInt(), 1);
     QCOMPARE(area->property("touchPointUpdateCount").toInt(), 0);
@@ -131,6 +127,7 @@ void tst_QQuickMultiPointTouchArea::signalTest()
     p1 -= QPoint(10,10);
     p2 += QPoint(10,10);
     sequence.move(0, p1).move(1, p2).stationary(2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(area->property("touchPointPressCount").toInt(), 0);
     QCOMPARE(area->property("touchPointUpdateCount").toInt(), 2);
@@ -141,6 +138,7 @@ void tst_QQuickMultiPointTouchArea::signalTest()
     p3 += QPoint(10,10);
     sequence.release(0, p1).release(1, p2)
             .move(2, p3).press(3, p4).press(4, p5).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(area->property("touchPointPressCount").toInt(), 2);
     QCOMPARE(area->property("touchPointUpdateCount").toInt(), 1);
@@ -149,6 +147,7 @@ void tst_QQuickMultiPointTouchArea::signalTest()
     QMetaObject::invokeMethod(area, "clearCounts");
 
     sequence.release(2, p3).release(3, p4).release(4, p5).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(area->property("touchPointPressCount").toInt(), 0);
     QCOMPARE(area->property("touchPointUpdateCount").toInt(), 0);
@@ -172,12 +171,14 @@ void tst_QQuickMultiPointTouchArea::release()
     QTest::QTouchEventSequence sequence = QTest::touchEvent(window.data(), device);
 
     sequence.press(0, p1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point1->pressed(), true);
 
     p1 += QPoint(0,10);
 
     sequence.move(0, p1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point1->pressed(), true);
     QCOMPARE(point1->x(), qreal(20)); QCOMPARE(point1->y(), qreal(110));
@@ -185,6 +186,7 @@ void tst_QQuickMultiPointTouchArea::release()
     p1 += QPoint(4,10);
 
     sequence.release(0, p1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     //test that a release without a prior move to the release position successfully updates the point's position
     QCOMPARE(point1->pressed(), false);
@@ -211,12 +213,14 @@ void tst_QQuickMultiPointTouchArea::reuse()
     QTest::QTouchEventSequence sequence = QTest::touchEvent(window.data(), device);
 
     sequence.press(0, p1).press(1, p2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point1->pressed(), true);
     QCOMPARE(point2->pressed(), true);
     QCOMPARE(point3->pressed(), false);
 
     sequence.release(0, p1).stationary(1).press(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     //we shouldn't reuse point 1 yet
     QCOMPARE(point1->pressed(), false);
@@ -225,24 +229,28 @@ void tst_QQuickMultiPointTouchArea::reuse()
 
     //back to base state (no touches)
     sequence.release(1, p2).release(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point1->pressed(), false);
     QCOMPARE(point2->pressed(), false);
     QCOMPARE(point3->pressed(), false);
 
     sequence.press(0, p1).press(1, p2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point1->pressed(), true);
     QCOMPARE(point2->pressed(), true);
     QCOMPARE(point3->pressed(), false);
 
     sequence.release(0, p1).stationary(1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point1->pressed(), false);
     QCOMPARE(point2->pressed(), true);
     QCOMPARE(point3->pressed(), false);
 
     sequence.press(4, p4).stationary(1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     //the new touch point should reuse point 1
     QCOMPARE(point1->pressed(), true);
@@ -278,6 +286,7 @@ void tst_QQuickMultiPointTouchArea::nonOverlapping()
     QTest::QTouchEventSequence sequence = QTest::touchEvent(window.data(), device);
 
     sequence.press(0, p1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), false);
     QCOMPARE(point12->pressed(), false);
@@ -286,6 +295,7 @@ void tst_QQuickMultiPointTouchArea::nonOverlapping()
     QCOMPARE(point23->pressed(), false);
 
     sequence.stationary(0).press(1, p2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -299,6 +309,7 @@ void tst_QQuickMultiPointTouchArea::nonOverlapping()
     p1 += QPoint(0,10);
     p2 += QPoint(5,0);
     sequence.move(0, p1).move(1, p2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -310,6 +321,7 @@ void tst_QQuickMultiPointTouchArea::nonOverlapping()
     QCOMPARE(point12->x(), qreal(45)); QCOMPARE(point12->y(), qreal(100));
 
     sequence.stationary(0).stationary(1).press(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -318,6 +330,7 @@ void tst_QQuickMultiPointTouchArea::nonOverlapping()
     QCOMPARE(point23->pressed(), false);
 
     sequence.stationary(0).stationary(1).stationary(2).press(3, p4).press(4, p5).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -337,6 +350,7 @@ void tst_QQuickMultiPointTouchArea::nonOverlapping()
     p4 += QPoint(1,-1);
     p5 += QPoint(-7,10);
     sequence.move(0, p1).move(1, p2).move(2, p3).move(3, p4).move(4, p5).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -351,6 +365,7 @@ void tst_QQuickMultiPointTouchArea::nonOverlapping()
     QCOMPARE(point23->x(), qreal(93)); QCOMPARE(point23->y(), qreal(30));
 
     sequence.release(0, p1).release(1, p2).release(2, p3).release(3, p4).release(4, p5).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), false);
     QCOMPARE(point12->pressed(), false);
@@ -383,6 +398,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     QTest::QTouchEventSequence sequence = QTest::touchEvent(window.data(), device);
 
     sequence.press(0, p1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), false);
     QCOMPARE(point12->pressed(), false);
@@ -391,6 +407,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     QCOMPARE(point23->pressed(), false);
 
     sequence.stationary(0).press(1, p2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -404,6 +421,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     p1 += QPoint(0,10);
     p2 += QPoint(5,0);
     sequence.move(0, p1).move(1, p2).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -415,6 +433,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     QCOMPARE(point12->x(), qreal(45)); QCOMPARE(point12->y(), qreal(100));
 
     sequence.stationary(0).stationary(1).press(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -430,6 +449,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     QCOMPARE(point23->x(), qreal(60)); QCOMPARE(point23->y(), qreal(180));
 
     sequence.stationary(0).stationary(1).stationary(2).press(3, QPoint(80,180)).press(4, QPoint(100,180)).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -450,6 +470,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     p2 += QPoint(17,17);
     p3 += QPoint(3,0);
     sequence.move(0, p1).move(1, p2).move(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -467,6 +488,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     p2 += QPoint(17,17);
     p3 += QPoint(3,0);
     sequence.move(0, p1).move(1, p2).move(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), false);
     QCOMPARE(point12->pressed(), false);
@@ -484,6 +506,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     sequence.release(0, p1).release(1, p2).release(2, p3).commit();
 
     sequence.press(0, p1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), false);
     QCOMPARE(point12->pressed(), false);
@@ -492,11 +515,13 @@ void tst_QQuickMultiPointTouchArea::nested()
     QCOMPARE(point23->pressed(), false);
 
     sequence.release(0, p1).commit();
+    QQuickTouchUtils::flush(window.data());
 
     //test with grabbing turned off
     window->rootObject()->setProperty("grabInnerArea", false);
 
     sequence.press(0, p1).press(1, p2).press(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -508,6 +533,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     p2 -= QPoint(17,17);
     p3 -= QPoint(3,0);
     sequence.move(0, p1).move(1, p2).move(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -525,6 +551,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     p2 -= QPoint(17,17);
     p3 -= QPoint(3,0);
     sequence.move(0, p1).move(1, p2).move(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -540,6 +567,7 @@ void tst_QQuickMultiPointTouchArea::nested()
     QCOMPARE(point23->x(), qreal(60)); QCOMPARE(point23->y(), qreal(180));
 
     sequence.release(0, p1).release(1, p2).release(2, p3).commit();
+    QQuickTouchUtils::flush(window.data());
 }
 
 void tst_QQuickMultiPointTouchArea::inFlickable()
@@ -550,6 +578,9 @@ void tst_QQuickMultiPointTouchArea::inFlickable()
     QQuickFlickable *flickable = qobject_cast<QQuickFlickable *>(window->rootObject());
     QVERIFY(flickable != 0);
 
+    QQuickMultiPointTouchArea *mpta = window->rootObject()->findChild<QQuickMultiPointTouchArea*>();
+    QVERIFY(mpta != 0);
+
     QQuickTouchPoint *point11 = window->rootObject()->findChild<QQuickTouchPoint*>("point1");
     QQuickTouchPoint *point12 = window->rootObject()->findChild<QQuickTouchPoint*>("point2");
 
@@ -561,25 +592,30 @@ void tst_QQuickMultiPointTouchArea::inFlickable()
 
     //moving one point vertically
     QTest::touchEvent(window.data(), device).press(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     QVERIFY(flickable->contentY() < 0);
     QCOMPARE(point11->pressed(), false);
     QCOMPARE(point12->pressed(), false);
 
     QTest::touchEvent(window.data(), device).release(0, p1);
-    QTest::qWait(50);
+    QQuickTouchUtils::flush(window.data());
 
     QTRY_VERIFY(!flickable->isMoving());
 
@@ -587,6 +623,7 @@ void tst_QQuickMultiPointTouchArea::inFlickable()
     p1 = QPoint(20,100);
     QTest::touchEvent(window.data(), device).press(0, p1).press(1, p2);
     QTest::mousePress(window.data(), Qt::LeftButton, 0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
@@ -596,18 +633,22 @@ void tst_QQuickMultiPointTouchArea::inFlickable()
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     QVERIFY(flickable->contentY() < 0);
     QCOMPARE(point11->pressed(), false);
@@ -617,7 +658,7 @@ void tst_QQuickMultiPointTouchArea::inFlickable()
 
     QTest::touchEvent(window.data(), device).release(0, p1).release(1, p2);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, p1);
-    QTest::qWait(50);
+    QQuickTouchUtils::flush(window.data());
 
     QTRY_VERIFY(!flickable->isMoving());
 
@@ -625,6 +666,9 @@ void tst_QQuickMultiPointTouchArea::inFlickable()
     p1 = QPoint(20,100);
     p2 = QPoint(40,100);
     QTest::touchEvent(window.data(), device).press(0, p1).press(1, p2);
+    QQuickTouchUtils::flush(window.data());
+    // ensure that mouse events do not fall through to the Flickable
+    mpta->setMaximumTouchPoints(3);
     QTest::mousePress(window.data(), Qt::LeftButton, 0, p1);
 
     QCOMPARE(point11->pressed(), true);
@@ -633,42 +677,50 @@ void tst_QQuickMultiPointTouchArea::inFlickable()
     p1 += QPoint(15,0); p2 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(15,0); p2 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(15,0); p2 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(15,0); p2 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15); p2 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1).move(1, p2);
     QTest::mouseMove(window.data(), p1);
+    QQuickTouchUtils::flush(window.data());
 
-    QVERIFY(flickable->contentY() == 0);
+    QCOMPARE(flickable->contentY(), qreal(0));
     QCOMPARE(point11->pressed(), true);
     QCOMPARE(point12->pressed(), true);
 
     QTest::touchEvent(window.data(), device).release(0, p1).release(1, p2);
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, p1);
-    QTest::qWait(50);
+    QQuickTouchUtils::flush(window.data());
 }
 
 // test that dragging out of a Flickable containing a MPTA doesn't harm Flickable's state.
@@ -689,28 +741,34 @@ void tst_QQuickMultiPointTouchArea::inFlickable2()
 
     // move point horizontally, out of Flickable area
     QTest::touchEvent(window.data(), device).press(0, p1);
+    QQuickTouchUtils::flush(window.data());
     QTest::mousePress(window.data(), Qt::LeftButton, 0, p1);
 
     p1 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
     QTest::mouseMove(window.data(), p1);
 
     p1 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
     QTest::mouseMove(window.data(), p1);
 
     p1 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
     QTest::mouseMove(window.data(), p1);
 
     p1 += QPoint(15,0);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
     QTest::mouseMove(window.data(), p1);
 
     QVERIFY(!flickable->isMoving());
     QVERIFY(point11->pressed());
 
     QTest::touchEvent(window.data(), device).release(0, p1);
+    QQuickTouchUtils::flush(window.data());
     QTest::mouseRelease(window.data(), Qt::LeftButton, 0, p1);
     QTest::qWait(50);
 
@@ -719,31 +777,287 @@ void tst_QQuickMultiPointTouchArea::inFlickable2()
     // Check that we can still move the Flickable
     p1 = QPoint(50,100);
     QTest::touchEvent(window.data(), device).press(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     QCOMPARE(point11->pressed(), true);
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     p1 += QPoint(0,15);
     QTest::touchEvent(window.data(), device).move(0, p1);
+    QQuickTouchUtils::flush(window.data());
 
     QVERIFY(flickable->contentY() < 0);
     QVERIFY(flickable->isMoving());
     QCOMPARE(point11->pressed(), true);
 
     QTest::touchEvent(window.data(), device).release(0, p1);
+    QQuickTouchUtils::flush(window.data());
     QTest::qWait(50);
 
     QTRY_VERIFY(!flickable->isMoving());
 }
 
+// QTBUG-31047
+void tst_QQuickMultiPointTouchArea::inMouseArea()
+{
+    QScopedPointer<QQuickView> window(createAndShowView("inMouseArea.qml"));
+    QVERIFY(window->rootObject() != 0);
+
+    QQuickMouseArea *mouseArea = qobject_cast<QQuickMouseArea *>(window->rootObject());
+    QVERIFY(mouseArea != 0);
+
+    QQuickMultiPointTouchArea *mpta = window->rootObject()->findChild<QQuickMultiPointTouchArea*>("mpta");
+    QVERIFY(mpta != 0);
+
+    QPoint innerPoint(40,100);
+    QPoint outerPoint(10,100);
+
+    QTest::touchEvent(window.data(), device).press(0, innerPoint);
+    QVERIFY(mpta->property("pressed").toBool());
+    QTest::touchEvent(window.data(), device).release(0, innerPoint);
+    QVERIFY(!mpta->property("pressed").toBool());
+
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, outerPoint);
+    QVERIFY(mouseArea->property("pressed").toBool());
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, outerPoint);
+    QVERIFY(!mouseArea->property("pressed").toBool());
+
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, innerPoint);
+    QVERIFY(mpta->property("pressed").toBool());
+    QVERIFY(!mouseArea->property("pressed").toBool());
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, innerPoint);
+    QVERIFY(!mpta->property("pressed").toBool());
+    QVERIFY(!mouseArea->property("pressed").toBool());
+
+    QTest::touchEvent(window.data(), device).press(0, innerPoint);
+    QVERIFY(mpta->property("pressed").toBool());
+    QTest::touchEvent(window.data(), device).release(0, innerPoint);
+    QVERIFY(!mpta->property("pressed").toBool());
+
+    QTest::touchEvent(window.data(), device).press(0, outerPoint);
+    QVERIFY(mouseArea->property("pressed").toBool());
+    QVERIFY(!mpta->property("pressed").toBool());
+    QTest::touchEvent(window.data(), device).release(0, outerPoint);
+    QVERIFY(!mouseArea->property("pressed").toBool());
+    QVERIFY(!mpta->property("pressed").toBool());
+
+    // Right click should pass through
+    QTest::mousePress(window.data(), Qt::RightButton, 0, innerPoint);
+    QVERIFY(mouseArea->property("pressed").toBool());
+    QVERIFY(!mpta->property("pressed").toBool());
+    QTest::mouseRelease(window.data(), Qt::RightButton, 0, innerPoint);
+
+    mpta->setProperty("mouseEnabled", false);
+
+    QTest::touchEvent(window.data(), device).press(0, innerPoint);
+    QVERIFY(mpta->property("pressed").toBool());
+    QVERIFY(!mouseArea->property("pressed").toBool());
+    QTest::touchEvent(window.data(), device).release(0, innerPoint);
+    QVERIFY(!mpta->property("pressed").toBool());
+    QVERIFY(!mouseArea->property("pressed").toBool());
+
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, outerPoint);
+    QVERIFY(mouseArea->property("pressed").toBool());
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, outerPoint);
+    QVERIFY(!mouseArea->property("pressed").toBool());
+
+    QTest::mousePress(window.data(), Qt::LeftButton, 0, innerPoint);
+    QVERIFY(!mpta->property("pressed").toBool());
+    QVERIFY(mouseArea->property("pressed").toBool());
+    QTest::mouseRelease(window.data(), Qt::LeftButton, 0, innerPoint);
+    QVERIFY(!mpta->property("pressed").toBool());
+    QVERIFY(!mouseArea->property("pressed").toBool());
+
+    QTest::touchEvent(window.data(), device).press(0, innerPoint);
+    QVERIFY(mpta->property("pressed").toBool());
+    QTest::touchEvent(window.data(), device).release(0, innerPoint);
+    QVERIFY(!mpta->property("pressed").toBool());
+
+    QTest::touchEvent(window.data(), device).press(0, outerPoint);
+    QVERIFY(mouseArea->property("pressed").toBool());
+    QVERIFY(!mpta->property("pressed").toBool());
+    QTest::touchEvent(window.data(), device).release(0, outerPoint);
+    QVERIFY(!mouseArea->property("pressed").toBool());
+    QVERIFY(!mpta->property("pressed").toBool());
+}
+
+void tst_QQuickMultiPointTouchArea::mouseAsTouchpoint()
+{
+    QScopedPointer<QQuickView> window(createAndShowView("dualGestures.qml"));
+    QVERIFY(window->rootObject() != 0);
+
+    QQuickMultiPointTouchArea *dualmpta = window->rootObject()->findChild<QQuickMultiPointTouchArea*>("dualTouchArea");
+    QVERIFY(dualmpta != 0);
+
+    QQuickItem *touch1rect = window->rootObject()->findChild<QQuickItem*>("touch1rect");
+    QQuickItem *touch2rect = window->rootObject()->findChild<QQuickItem*>("touch2rect");
+    QQuickItem *touch3rect = window->rootObject()->findChild<QQuickItem*>("touch3rect");
+    QQuickItem *touch4rect = window->rootObject()->findChild<QQuickItem*>("touch4rect");
+    QQuickItem *touch5rect = window->rootObject()->findChild<QQuickItem*>("touch5rect");
+
+    {
+        QPoint touch1(40,10);
+        QPoint touch2(40,100);
+        QPoint touch3(10,10);
+
+        // Touch both, release one, manipulate other touchpoint with mouse
+        QTest::touchEvent(window.data(), device).press(1, touch1);
+        QQuickTouchUtils::flush(window.data());
+        QTest::touchEvent(window.data(), device).press(2, touch2);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+        QTest::touchEvent(window.data(), device).release(1, touch1);
+        touch1.setY(20);
+        QTest::mousePress(window.data(), Qt::LeftButton, 0, touch1);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+        QTest::touchEvent(window.data(), device).release(2, touch2);
+        QTest::mouseRelease(window.data(), Qt::LeftButton, 0, touch1);
+        QQuickTouchUtils::flush(window.data());
+
+        // Start with mouse, move it, touch second point, move it
+        QTest::mousePress(window.data(), Qt::LeftButton, 0, touch1);
+        touch1.setX(60);
+        QTest::mouseMove(window.data(), touch1);
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        touch2.setX(60);
+        QTest::touchEvent(window.data(), device).press(3, touch2);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+        touch2.setY(150);
+        QTest::touchEvent(window.data(), device).move(3, touch2);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+
+        // Touch third point - nothing happens
+        QTest::touchEvent(window.data(), device).press(4, touch3);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+
+        // Release all
+        QTest::mouseRelease(window.data(), Qt::LeftButton, 0, touch1);
+        QTest::touchEvent(window.data(), device).release(3, touch2);
+        QQuickTouchUtils::flush(window.data());
+        QTest::touchEvent(window.data(), device).release(4, touch3);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+    }
+
+    // Mouse and touch on left, touch 3 points on right
+    {
+        QPoint mouse1(40,10);
+        QPoint touch1(10,10);
+        QPoint touch2(340,10);
+        QPoint touch3(340,100);
+        QPoint touch4(540,10);
+
+        QTest::mousePress(window.data(), Qt::LeftButton, 0, mouse1);
+        QCOMPARE(touch1rect->property("x").toInt(), mouse1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), mouse1.y());
+        QTest::touchEvent(window.data(), device).press(1, touch1);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), mouse1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), mouse1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch1.y());
+
+        QTest::touchEvent(window.data(), device).press(2, touch2).press(3, touch3).press(4, touch4);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), mouse1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), mouse1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch3rect->property("x").toInt(), touch2.x() - 320);
+        QCOMPARE(touch3rect->property("y").toInt(), touch2.y());
+        QCOMPARE(touch4rect->property("x").toInt(), touch3.x() - 320);
+        QCOMPARE(touch4rect->property("y").toInt(), touch3.y());
+        QCOMPARE(touch5rect->property("x").toInt(), touch4.x() - 320);
+        QCOMPARE(touch5rect->property("y").toInt(), touch4.y());
+
+        // Release all
+        QTest::mouseRelease(window.data(), Qt::LeftButton, 0, mouse1);
+        QTest::touchEvent(window.data(), device).release(1, touch1).release(2, touch2).release(3, touch3).release(4, touch4);
+        QQuickTouchUtils::flush(window.data());
+    }
+
+    dualmpta->setProperty("mouseEnabled", false);
+    {
+        QPoint mouse1(40,10);
+        QPoint touch1(10,10);
+        QPoint touch2(100,10);
+
+        touch1rect->setX(10);
+        touch1rect->setY(10);
+        touch2rect->setX(20);
+        touch2rect->setY(10);
+
+        // Start with mouse, move it, touch a point, move it, touch another.
+        // Mouse is ignored, both touch points are heeded.
+        QTest::mousePress(window.data(), Qt::LeftButton, 0, mouse1);
+        mouse1.setX(60);
+        QTest::mouseMove(window.data(), mouse1);
+        QCOMPARE(touch1rect->property("x").toInt(), 10);
+        QCOMPARE(touch1rect->property("y").toInt(), 10);
+
+        QTest::touchEvent(window.data(), device).press(1, touch1);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        touch1.setY(150);
+        QTest::touchEvent(window.data(), device).move(1, touch1);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QTest::touchEvent(window.data(), device).press(2, touch2);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+
+        // Release all
+        QTest::mouseRelease(window.data(), Qt::LeftButton, 0, mouse1);
+        QTest::touchEvent(window.data(), device).release(1, touch1);
+        QQuickTouchUtils::flush(window.data());
+        QTest::touchEvent(window.data(), device).release(2, touch2);
+        QQuickTouchUtils::flush(window.data());
+        QCOMPARE(touch1rect->property("x").toInt(), touch1.x());
+        QCOMPARE(touch1rect->property("y").toInt(), touch1.y());
+        QCOMPARE(touch2rect->property("x").toInt(), touch2.x());
+        QCOMPARE(touch2rect->property("y").toInt(), touch2.y());
+    }
+}
+
 // QTBUG-23327
 void tst_QQuickMultiPointTouchArea::invisible()
 {
@@ -785,7 +1099,6 @@ void tst_QQuickMultiPointTouchArea::transformedTouchArea_data()
 
     QTest::newRow("3rd point inside")
         << QPoint(140, 260) << QPoint(260, 140) << QPoint(200, 140) << 0 << 0 << 1;
-
     QTest::newRow("all points inside")
         << QPoint(200, 140) << QPoint(200, 260) << QPoint(140, 200) << 1 << 2 << 3;
 
@@ -841,6 +1154,48 @@ QQuickView *tst_QQuickMultiPointTouchArea::createAndShowView(const QString &file
     return window;
 }
 
+void tst_QQuickMultiPointTouchArea::mouseInteraction_data()
+{
+    QTest::addColumn<int>("buttons");
+    QTest::addColumn<int>("accept");
+
+    QTest::newRow("left") << (int) Qt::LeftButton << 1;
+    QTest::newRow("right") << (int) Qt::RightButton << 0;
+    QTest::newRow("middle") << (int) Qt::MiddleButton << 0;
+}
+
+void tst_QQuickMultiPointTouchArea::mouseInteraction()
+{
+    QFETCH(int, buttons);
+    QFETCH(int, accept);
+
+    QScopedPointer<QQuickView> view(createAndShowView("mouse.qml"));
+    QVERIFY(view->rootObject() != 0);
+
+    QQuickMultiPointTouchArea *area = qobject_cast<QQuickMultiPointTouchArea *>(view->rootObject());
+    QVERIFY(area != 0);
+    QQuickTouchPoint *point1 = view->rootObject()->findChild<QQuickTouchPoint*>("point1");
+    QCOMPARE(point1->pressed(), false);
+
+    QCOMPARE(area->property("touchCount").toInt(), 0);
+    QPoint p1 = QPoint(100, 100);
+    QTest::mousePress(view.data(), (Qt::MouseButton) buttons, 0, p1);
+    QCOMPARE(area->property("touchCount").toInt(), accept);
+    QCOMPARE(point1->pressed(), accept != 0);
+    p1 += QPoint(10, 10);
+    QTest::mouseMove(view.data(), p1);
+    QCOMPARE(point1->pressed(), accept != 0);
+    QCOMPARE(area->property("touchCount").toInt(), accept);
+    p1 += QPoint(10, 10);
+    QTest::mouseMove(view.data(), p1);
+    QCOMPARE(point1->pressed(), accept != 0);
+    QCOMPARE(area->property("touchCount").toInt(), accept);
+    QTest::mouseRelease(view.data(), (Qt::MouseButton) buttons);
+    QCOMPARE(point1->pressed(), false);
+    QCOMPARE(area->property("touchCount").toInt(), 0);
+}
+
+
 QTEST_MAIN(tst_QQuickMultiPointTouchArea)
 
 #include "tst_qquickmultipointtoucharea.moc"
