@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -351,6 +346,7 @@ private slots:
     void crashWhenHoverItemDeleted();
 
     void unloadSubWindow();
+    void changeVisibilityInCompleted();
 
     void qobjectEventFilter_touch();
     void qobjectEventFilter_key();
@@ -397,7 +393,7 @@ void tst_qquickwindow::openglContextCreatedSignal()
     QVERIFY(spy.size() > 0);
 
     QVariant ctx = spy.at(0).at(0);
-    QCOMPARE(qVariantValue<QOpenGLContext *>(ctx), window.openglContext());
+    QCOMPARE(qvariant_cast<QOpenGLContext *>(ctx), window.openglContext());
 }
 
 void tst_qquickwindow::aboutToStopSignal()
@@ -1852,6 +1848,28 @@ void tst_qquickwindow::unloadSubWindow()
     QTRY_VERIFY(transient.isNull() || !transient->isVisible());
 }
 
+// QTBUG-52573
+void tst_qquickwindow::changeVisibilityInCompleted()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.loadUrl(testFileUrl("changeVisibilityInCompleted.qml"));
+    QScopedPointer<QQuickWindow> window(qobject_cast<QQuickWindow *>(component.create()));
+    QVERIFY(!window.isNull());
+    window->setTitle(QTest::currentTestFunction());
+    window->show();
+    QTest::qWaitForWindowExposed(window.data());
+    QPointer<QQuickWindow> winVisible;
+    QTRY_VERIFY(winVisible = window->property("winVisible").value<QQuickWindow*>());
+    QPointer<QQuickWindow> winVisibility;
+    QTRY_VERIFY(winVisibility = window->property("winVisibility").value<QQuickWindow*>());
+    QTest::qWaitForWindowExposed(winVisible);
+    QTest::qWaitForWindowExposed(winVisibility);
+
+    QVERIFY(winVisible->isVisible());
+    QCOMPARE(winVisibility->visibility(), QWindow::Windowed);
+}
+
 // QTBUG-32004
 void tst_qquickwindow::qobjectEventFilter_touch()
 {
@@ -2038,6 +2056,7 @@ void tst_qquickwindow::attachedProperty()
     QCOMPARE(view.rootObject()->property("contentItem").value<QQuickItem*>(), view.contentItem());
     QCOMPARE(view.rootObject()->property("windowWidth").toInt(), view.width());
     QCOMPARE(view.rootObject()->property("windowHeight").toInt(), view.height());
+    QCOMPARE(view.rootObject()->property("window").value<QQuickView*>(), &view);
 
     QQuickWindow *innerWindow = view.rootObject()->findChild<QQuickWindow*>("extraWindow");
     QVERIFY(innerWindow);
@@ -2050,11 +2069,13 @@ void tst_qquickwindow::attachedProperty()
     QCOMPARE(text->property("contentItem").value<QQuickItem*>(), innerWindow->contentItem());
     QCOMPARE(text->property("windowWidth").toInt(), innerWindow->width());
     QCOMPARE(text->property("windowHeight").toInt(), innerWindow->height());
+    QCOMPARE(text->property("window").value<QQuickWindow*>(), innerWindow);
 
     text->setParentItem(0);
     QVERIFY(!text->property("contentItem").value<QQuickItem*>());
     QCOMPARE(text->property("windowWidth").toInt(), 0);
     QCOMPARE(text->property("windowHeight").toInt(), 0);
+    QVERIFY(!text->property("window").value<QQuickWindow*>());
 }
 
 class RenderJob : public QRunnable
