@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -44,7 +39,6 @@
 #include <private/qsgcontext_p.h>
 #include <private/qsgrenderloop_p.h>
 
-#include "../../shared/util.h"
 
 class PerPixelRect : public QQuickItem
 {
@@ -90,7 +84,7 @@ private:
     QColor m_color;
 };
 
-class tst_SceneGraph : public QQmlDataTest
+class tst_SceneGraph : public QObject
 {
     Q_OBJECT
 
@@ -110,7 +104,6 @@ private slots:
 
 private:
     bool m_brokenMipmapSupport;
-    QQuickView *createView(const QString &file, QWindow *parent = 0, int x = -1, int y = -1, int w = -1, int h = -1);
 };
 
 template <typename T> class ScopedList : public QList<T> {
@@ -122,8 +115,6 @@ void tst_SceneGraph::initTestCase()
 {
     qmlRegisterType<PerPixelRect>("SceneGraphTest", 1, 0, "PerPixelRect");
 
-    QQmlDataTest::initTestCase();
-
     QSGRenderLoop *loop = QSGRenderLoop::instance();
     qDebug() << "RenderLoop:        " << loop;
 
@@ -161,16 +152,26 @@ void tst_SceneGraph::initTestCase()
     context.doneCurrent();
 }
 
-QQuickView *tst_SceneGraph::createView(const QString &file, QWindow *parent, int x, int y, int w, int h)
+QQuickView *createView(const QString &file, QWindow *parent = 0, int x = -1, int y = -1, int w = -1, int h = -1)
 {
     QQuickView *view = new QQuickView(parent);
-    view->setSource(testFileUrl(file));
+    view->setSource(QUrl::fromLocalFile("data/" + file));
     if (x >= 0 && y >= 0) view->setPosition(x, y);
     if (w >= 0 && h >= 0) view->resize(w, h);
     view->show();
     return view;
 }
 
+QImage showAndGrab(const QString &file, int w, int h)
+{
+    QQuickView view;
+    view.setSource(QUrl::fromLocalFile(QStringLiteral("data/") + file));
+    if (w >= 0 && h >= 0)
+        view.resize(w, h);
+    view.create();
+    return view.grabWindow();
+}
+
 // Assumes the images are opaque white...
 bool containsSomethingOtherThanWhite(const QImage &image)
 {
@@ -404,17 +405,17 @@ void tst_SceneGraph::render_data()
     QTest::addColumn<QList<Sample> >("finalStage");
 
     QList<QString> files;
-    files << "render_DrawSets.qml"
-          << "render_Overlap.qml"
-          << "render_MovingOverlap.qml"
-          << "render_BreakOpacityBatch.qml"
-          << "render_OutOfFloatRange.qml"
-          << "render_StackingOrder.qml"
-          << "render_ImageFiltering.qml"
-          << "render_bug37422.qml"
-          << "render_OpacityThroughBatchRoot.qml";
+    files << "data/render_DrawSets.qml"
+          << "data/render_Overlap.qml"
+          << "data/render_MovingOverlap.qml"
+          << "data/render_BreakOpacityBatch.qml"
+          << "data/render_OutOfFloatRange.qml"
+          << "data/render_StackingOrder.qml"
+          << "data/render_ImageFiltering.qml"
+          << "data/render_bug37422.qml"
+          << "data/render_OpacityThroughBatchRoot.qml";
     if (!m_brokenMipmapSupport)
-          files << "render_Mipmap.qml";
+          files << "data/render_Mipmap.qml";
 
     QRegExp sampleCount("#samples: *(\\d+)");
     //                          X:int   Y:int   R:float       G:float       B:float       Error:float
@@ -422,7 +423,7 @@ void tst_SceneGraph::render_data()
     QRegExp finalSamples("#final: *(\\d+) *(\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+) *(\\d\\.\\d+)");
 
     foreach (QString fileName, files) {
-        QFile file(testFile(fileName));
+        QFile file(fileName);
         if (!file.open(QFile::ReadOnly)) {
             qFatal("render_data: QFile::open failed! file=%s, error=%s",
                    qPrintable(fileName), qPrintable(file.errorString()));
@@ -465,7 +466,7 @@ void tst_SceneGraph::render()
 
     QQuickView view;
     view.rootContext()->setContextProperty("suite", &suite);
-    view.setSource(testFileUrl(file));
+    view.setSource(QUrl::fromLocalFile(file));
     view.setResizeMode(QQuickView::SizeViewToRootObject);
     view.show();
     QVERIFY(QTest::qWaitForWindowExposed(&view));
@@ -513,7 +514,7 @@ void tst_SceneGraph::hideWithOtherContext()
 
     {
         QQuickView view;
-        view.setSource(testFileUrl("simple.qml"));
+        view.setSource(QUrl::fromLocalFile("data/simple.qml"));
         view.setResizeMode(QQuickView::SizeViewToRootObject);
         view.show();
         QVERIFY(QTest::qWaitForWindowExposed(&view));
