@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -51,12 +38,11 @@
 // Timeout values:
 
 // A valid screen grab requires the scene to not change
-// for SCENE_STABLE_TIME ms (default 500)
-#define SCENE_STABLE_TIME  500
+// for SCENE_STABLE_TIME ms
+#define SCENE_STABLE_TIME 200
 
 // Give up after SCENE_TIMEOUT ms
-#define SCENE_TIMEOUT     16000
-
+#define SCENE_TIMEOUT     6000
 
 //#define GRABBERDEBUG
 
@@ -66,30 +52,44 @@ class GrabbingView : public QQuickView
 
 public:
     GrabbingView(const QString &outputFile)
-        : ofile(outputFile), frames(0), isGrabbing(false)
+        : ofile(outputFile), grabNo(0), isGrabbing(false), initDone(false)
     {
-        connect(this, SIGNAL(afterRendering()), SLOT(renderingDone()));
+        grabTimer = new QTimer(this);
+        grabTimer->setSingleShot(true);
+        grabTimer->setInterval(SCENE_STABLE_TIME);
+        connect(grabTimer, SIGNAL(timeout()), SLOT(grab()));
+
+        connect(this, SIGNAL(afterRendering()), SLOT(startGrabbing()));
+
         QTimer::singleShot(SCENE_TIMEOUT, this, SLOT(timedOut()));
-        stableSceneTimer.setSingleShot(true);
-        connect(&stableSceneTimer, SIGNAL(timeout()), SLOT(sceneStabilized()));
     }
 
 private slots:
-    void renderingDone()
+    void startGrabbing()
+    {
+        if (!initDone) {
+            initDone = true;
+            grabTimer->start();
+        }
+    }
+
+    void grab()
     {
         if (isGrabbing)
             return;
         isGrabbing = true;
-        frames++;
+        grabNo++;
 #ifdef GRABBERDEBUG
-        printf("...frame %i\n", frames);
+        printf("grab no. %i\n", grabNo);
 #endif
         QImage img = grabWindow();
-        //qDebug() << "Rendering done, grab is" << !img.isNull() << "timer valid:" << stableSceneTimer.isActive()  << "same as last:" << (img == lastGrab);
-        if (!img.isNull() && img != lastGrab) {
+        if (!img.isNull() && img == lastGrab) {
+            sceneStabilized();
+        } else {
             lastGrab = img;
-            stableSceneTimer.start(SCENE_STABLE_TIME);
+            grabTimer->start();
         }
+
         isGrabbing = false;
     }
 
@@ -112,7 +112,6 @@ private slots:
                 return;
             }
         }
-
         QGuiApplication::exit(0);
 #ifdef GRABBERDEBUG
         printf("...sceneStabilized OUT\n");
@@ -121,16 +120,17 @@ private slots:
 
     void timedOut()
     {
-        qWarning() << "Error: timed out waiting for scene to stabilize." << frames << "frame(s) received. Last grab was" << (lastGrab.isNull() ? "invalid." : "valid.");
+        qWarning() << "Error: timed out waiting for scene to stabilize." << grabNo << "grab(s) done. Last grab was" << (lastGrab.isNull() ? "invalid." : "valid.");
         QGuiApplication::exit(3);
     }
 
 private:
     QImage lastGrab;
-    QTimer stableSceneTimer;
+    QTimer *grabTimer;
     QString ofile;
-    int frames;
+    int grabNo;
     bool isGrabbing;
+    bool initDone;
 };
 
 
